@@ -1,10 +1,24 @@
 """
 Infrastructure to define optimization problems.
 """
-
 import math
 import copy
 import multiprocessing.dummy
+from collections import OrderedDict
+
+
+class Aborted(StopIteration):
+    """Indicates abortion of optimization.
+
+    May be due to an unpreventable problem without default treatment,
+    e.g., a numerical problem. Inherits from :class:`StopIteration`.
+
+    """
+    def __init__(self, message=None):
+        message_parts = ["Optimization aborted"]
+        if message is not None:
+            message_parts.extend([". Reason: ", message])
+        StopIteration.__init__(self, "".join(message_parts))
 
 
 
@@ -22,19 +36,29 @@ class ResourcesExhausted(StopIteration):
 
 
 
-class BoundConstraintError(ValueError):
-    """Used to report violations of bound constraints.
+class Solved(StopIteration):
+    """Indicates the problem has been satisfactorily solved.
 
-    Inherits from ValueError.
+    Inherits from :class:`StopIteration`.
 
     """
-    def __init__(self, value, min_bound, max_bound, variable_name=None):
-        bounds = "[" + str(min_bound) + ", " + str(max_bound) + "]"
-        if variable_name is None:
-            message = "Value " + str(value) + " out of bounds " + bounds
-        else:
-            message = "Variable " + variable_name + " (=" + str(value) + ") out of bounds " + bounds
-        ValueError.__init__(self, message)
+    def __init__(self, found_solution=None):
+        self.found_solution = found_solution
+        StopIteration.__init__(self, "Problem solved")
+
+
+
+class Stalled(StopIteration):
+    """Indicates stagnation of optimization (not an error).
+
+    Inherits from :class:`StopIteration`.
+
+    """
+    def __init__(self, criterion=None):
+        message_parts = ["Optimization has stalled"]
+        if criterion is not None:
+            message_parts.extend([". Criterion: ", str(criterion)])
+        StopIteration.__init__(self, "".join(message_parts))
 
 
 
@@ -58,8 +82,9 @@ class Individual:
         phenome : object, optional
             An arbitrary object which somehow can be evaluated by an
             objective function.
-        objective_values : list, optional
-            A list of objective values.
+        objective_values : float or list, optional
+            A single float or a list of objective values.
+
         """
         self.phenome = phenome
         self.objective_values = objective_values
@@ -94,12 +119,12 @@ class BundledObjectives:
 
 
 
-def identity(phenome):
-    return phenome
+def identity(argument):
+    return argument
 
 
 
-class Problem:
+class Problem(object):
     """The base class for problems to be solved.
 
     In the simplest case you can use this class directly by wrapping
@@ -110,7 +135,7 @@ class Problem:
     def __init__(self, objective_functions,
                  num_objectives=None,
                  max_evaluations=float("inf"),
-                 num_processes=1,
+                 worker_pool=None,
                  mp_module=None,
                  phenome_preprocessor=None,
                  name=None):
@@ -130,12 +155,16 @@ class Problem:
         max_evaluations : int, optional
             The maximum budget of function evaluations. By default there
             is no restriction.
-        num_processes : int, optional
-            The number of parallel processes. Must be >0. Default is 1
-            (no parallelization).
+        worker_pool : Pool, optional
+            A pool of worker processes. Default is None (no 
+            parallelization).
         mp_module : module, optional
             Either `multiprocessing`, `multiprocessing.dummy` (default),
-            or a `MockMultiProcessing` instance.
+            or a `MockMultiProcessing` instance. This is only used to create
+            an internal lock around bookkeeping code in various places. The
+            lock is only required for asynchronous parallelization, but not
+            for the parallelization with a worker pool in 
+            :func:`batch_evaluate`.
         phenome_preprocessor : callable, optional
             A callable potentially applying transformations or checks to
             the phenome. Modifications should only be applied to a copy
@@ -164,8 +193,7 @@ class Problem:
         self.num_objectives = num_objectives
         self.remaining_evaluations = max_evaluations
         self.consumed_evaluations = 0
-        assert num_processes > 0
-        self.num_processes = num_processes
+        self.worker_pool = worker_pool
         self.chunksize = 1
         if mp_module is None:
             mp_module = multiprocessing.dummy
@@ -204,7 +232,11 @@ class Problem:
             else:
                 raise ResourcesExhausted("problem evaluations")
         objective_values = self.objective_function(phenome)
-        assert len(objective_values) == self.num_objectives
+        try:
+            num_obj_values = len(objective_values)
+        except TypeError:
+            num_obj_values = 1
+        assert num_obj_values == self.num_objectives
         return objective_values
 
 
@@ -233,24 +265,29 @@ class Problem:
             this exception is thrown.
 
         """
-        if self.num_processes == 1 or len(individuals) == 1:
+        if self.worker_pool is None or len(individuals) == 1:
             evaluate = self.evaluate
             for individual in individuals:
                 evaluate(individual)
         else:
             preprocessor = self.phenome_preprocessor
             num_objectives = self.num_objectives
-            worker_pool = self.mp_module.Pool(self.num_processes)
             with self.lock:
                 budgeted_evaluations = min(len(individuals), self.remaining_evaluations)
                 affordable_individuals = individuals[:budgeted_evaluations]
                 phenomes = [preprocessor(ind.phenome) for ind in affordable_individuals]
                 self.consumed_evaluations += budgeted_evaluations
                 self.remaining_evaluations -= budgeted_evaluations
-            results = worker_pool.map(self.objective_function, phenomes, chunksize=self.chunksize)
-            for individual, result in zip(individuals, results):
-                assert len(result) == num_objectives
-                individual.objective_values = result
+            results = self.worker_pool.map(self.objective_function,
+                                           phenomes,
+                                           chunksize=self.chunksize)
+            for individual, objective_values in zip(individuals, results):
+                try:
+                    num_obj_values = len(objective_values)
+                except TypeError:
+                    num_obj_values = 1
+                assert num_obj_values == num_objectives
+                individual.objective_values = objective_values
             if len(individuals) > len(affordable_individuals):
                 raise ResourcesExhausted("problem evaluations")
 
@@ -274,6 +311,14 @@ class MockMultiProcessing:
         return list(map(func, iterable))
 
 
+    def close(self):
+        pass
+
+
+    def join(self):
+        pass
+
+
     def __enter__(self):
         return self
 
@@ -327,7 +372,7 @@ class Cache(Problem):
         self.problem = problem
         self.hash_function = hash_function
         self.saved_evaluations = 0
-        self.archive = dict()
+        self.archive = OrderedDict()
 
 
     @staticmethod
@@ -335,15 +380,23 @@ class Cache(Problem):
         """Guess if a variable contains valid objective values.
 
         Objective values are only stored in the archive if this method
-        returns True. In particular, None and empty lists are considered
-        invalid. So, you may have to override this method if you have very
-        exotic objective values.
+        returns True. In particular, None, strings, and empty lists are
+        considered invalid. A list of arbitrary objects except for None
+        and the empty list is considered valid, however. So, you may have
+        to override this method if you have very exotic objective values.
 
         """
         valid = objective_values is not None
-        valid = valid and len(objective_values) > 0
-        valid = valid and None not in objective_values
-        valid = valid and [] not in objective_values
+        try:
+            num_obj_values = len(objective_values)
+        except TypeError:
+            return valid
+        valid = valid and num_obj_values > 0
+        try:
+            valid = valid and None not in objective_values
+            valid = valid and [] not in objective_values
+        except TypeError:
+            return False
         return valid
 
 
@@ -353,8 +406,8 @@ class Cache(Problem):
 
 
     @property
-    def num_processes(self):
-        return self.problem.num_processes
+    def worker_pool(self):
+        return self.problem.worker_pool
 
 
     @property
@@ -362,11 +415,21 @@ class Cache(Problem):
         return self.problem.consumed_evaluations
 
 
+    @consumed_evaluations.setter
+    def consumed_evaluations(self, value):
+        self.problem.consumed_evaluations = value
+
+
     @property
     def remaining_evaluations(self):
         return self.problem.remaining_evaluations
 
 
+    @remaining_evaluations.setter
+    def remaining_evaluations(self, value):
+        self.problem.remaining_evaluations = value
+
+
     def __str__(self):
         """Return the name of this problem."""
         return "Cached " + str(self.problem)
@@ -402,12 +465,12 @@ class Cache(Problem):
         delegated to the original problem. Finally, the new evaluations are
         stored in the archive, if they are valid.
 
-        .. warning:: If ``num_processes > 1``, duplicates in a batch will
-                    not be recognized, leading to an expensive function
+        .. warning:: If ``worker_pool is not None``, duplicates in a batch
+                    will not be recognized, leading to an expensive function
                     evaluation for each one.
 
         """
-        if self.num_processes == 1 or len(individuals) == 1:
+        if self.worker_pool is None or len(individuals) == 1:
             evaluate = self.evaluate
             for individual in individuals:
                 evaluate(individual)
@@ -427,12 +490,9 @@ class Cache(Problem):
             try:
                 self.problem.batch_evaluate(unrecognized_individuals)
             finally:
+                are_objectives_valid = self.are_objectives_valid
                 for individual in unrecognized_individuals:
-                    valid = False
-                    try:
-                        valid = individual.are_objectives_valid()
-                    except StandardError:
-                        valid = self.are_objectives_valid(individual.objective_values)
+                    valid = are_objectives_valid(individual.objective_values)
                     if valid:
                         key = hash_function(individual.phenome)
                         with self.problem.lock:
@@ -440,6 +500,93 @@ class Cache(Problem):
 
 
 
+class ScalingPreprocessor:
+    """Translation and linear transformation between arbitrary cuboids.
+
+    This class is useful to normalize the search space, while the problem can
+    keep using its "native" units. If there are infinities in the bounds,
+    values in the corresponding dimension are not scaled.
+
+    .. note:: This class makes use of the decorator design pattern for
+        potential chaining of pre-processors, see
+        https://en.wikipedia.org/wiki/Decorator_pattern
+
+    """
+    def __init__(self, from_cuboid, to_cuboid, previous_preprocessor=None):
+        """Constructor.
+
+        Parameters
+        ----------
+        from_cuboid : tuple of sequence
+            This is the space an optimization algorithm would use. The first
+            sequence contains the lower bounds, the second sequence
+            contains the upper bounds for each variable.
+        to_cuboid : tuple of sequence
+            This is the domain of the optimization problem. The first
+            sequence contains the lower bounds, the second sequence contains
+            the upper bounds for each variable.
+        previous_preprocessor : callable, optional
+            Another callable that processes the phenome before this one
+            does.
+
+        """
+        self.from_cuboid = from_cuboid
+        self.to_cuboid = to_cuboid
+        min_bounds_to, max_bounds_to = to_cuboid
+        assert len(min_bounds_to) == len(max_bounds_to)
+        for min_bound_to, max_bound_to in zip(min_bounds_to, max_bounds_to):
+            assert max_bound_to >= min_bound_to
+        min_bounds_from, max_bounds_from = from_cuboid
+        assert len(min_bounds_from) == len(max_bounds_from)
+        for min_bound_from, max_bound_from in zip(min_bounds_from, max_bounds_from):
+            assert max_bound_from >= min_bound_from
+        self.previous_preprocessor = previous_preprocessor
+
+
+    def __call__(self, phenome):
+        """Transform from one space to the other.
+
+        This function does not check if the phenome is actually inside
+        `from_cuboid`.
+
+        """
+        if self.previous_preprocessor is not None:
+            phenome = self.previous_preprocessor(phenome)
+        min_bounds_to, max_bounds_to = self.to_cuboid
+        min_bounds_from, max_bounds_from = self.from_cuboid
+        assert len(phenome) == len(min_bounds_from)
+        # scale
+        scaled_point = copy.copy(phenome)
+        for i, value in enumerate(phenome):
+            max_to = max_bounds_to[i]
+            min_to = min_bounds_to[i]
+            max_from = max_bounds_from[i]
+            min_from = min_bounds_from[i]
+            if not any(math.isinf(bound) for bound in (max_to, min_to, max_from, min_from)):
+                length_factor = max_to - min_to
+                length_factor /= max_from - min_from
+                scaled_point[i] = (value - min_from) * length_factor
+                scaled_point[i] += min_to
+        return scaled_point
+
+
+
+class BoundConstraintError(ValueError):
+    """Used to report violations of bound constraints.
+
+    Inherits from :class:`ValueError`.
+
+    """
+    def __init__(self, value, min_bound, max_bound, variable_name=None):
+        bounds = "[" + str(min_bound) + ", " + str(max_bound) + "]"
+        if variable_name is None:
+            message = "Value " + str(value) + " out of bounds " + bounds
+        else:
+            message = "Variable " + variable_name + " (=" + str(value) + ") out of bounds " + bounds
+        ValueError.__init__(self, message)
+
+
+
 def min_bound_violated(value, min_bound):
     """Return True if ``min_bound != None and value < min_bound``."""
     return min_bound is not None and value < min_bound
@@ -487,11 +634,11 @@ class BoundConstraintsChecker:
             If any bound constraint is violated.
 
         """
+        if self.previous_preprocessor is not None:
+            phenome = self.previous_preprocessor(phenome)
         max_bounds = self.max_bounds
         min_bounds = self.min_bounds
         assert len(phenome) == len(min_bounds)
-        if self.previous_preprocessor is not None:
-            phenome = self.previous_preprocessor(phenome)
         for i, phene in enumerate(phenome):
             if min_bound_violated(phene, min_bounds[i]) or max_bound_violated(phene, max_bounds[i]):
                 raise BoundConstraintError(phene, min_bounds[i], max_bounds[i], variable_name="x"+str(i))
@@ -590,7 +737,6 @@ def wrap(value, min_bound, max_bound):
         The repaired decision variable.
 
     """
-
     if max_bound is None or min_bound is None:
         raise Exception("Wrapping is only applicable if lower and upper bounds exist")
     if math.isinf(value):
@@ -639,7 +785,9 @@ class BoundConstraintsRepair:
         repair_modes : sequence
             Contains an individual repair mode for each variable. The
             methods projection, reflection, and wrapping are supported.
-            Values of None lead to raised exceptions.
+            Values of None indicate that repair is not possible/desired.
+            If a constraint violation is detected in this case, an
+            exception is raised.
         previous_preprocessor : callable, optional
             Another callable that processes the phenome before this one
             does.
