@@ -1,5 +1,5 @@
 """
-This module contains the second version of the multiple peaks model (MPM).
+This module contains the second version of the multiple-peaks model (MPM).
 This test problem can be parametrized in many different ways. It is also
 able to enumerate all local optima and to (approximately) determine which
 attraction basin a point is located in. These features make it especially
@@ -16,7 +16,11 @@ from optproblems.base import TestProblem, Individual, reflect
 
 
 class Peak(list):
-    """Helper class maintaining the data structures needed for one peak."""
+    """Helper class maintaining the data structures needed for one peak.
+
+    Inherits from :class:`list`. The stored data is the position.
+
+    """
 
     def __init__(self,
                  num_variables,
@@ -116,9 +120,9 @@ class MultiplePeaksModel2(TestProblem):
         peaks : sequence of Peak
             Previously prepared peaks. If None, a few peaks are generated
             randomly.
-        **kwargs
-            Arbitrary keyword arguments, delegated to the constructor of
-            the super class.
+        kwargs
+            Arbitrary keyword arguments, passed through to the constructor
+            of the super class.
 
         """
         self.min_bounds = [0.0] * num_variables
@@ -174,12 +178,12 @@ class MultiplePeaksModel2(TestProblem):
         radius_range : tuple of float, optional
             The exponentiated Mahalanobis distance is divided by a value
             drawn random uniformly from this range.
-        rotated : bool, optional
+        rotated_peaks : bool, optional
             If true, a random rotation is applied to each peak.
 
         Returns
         -------
-        problem : MultiplePeaksModel2 instance
+        problem : MultiplePeaksModel2
 
         """
         if height_range is None:
@@ -274,10 +278,12 @@ class MultiplePeaksModel2(TestProblem):
             Must be -1, 0, or 1. If this value is not zero, also the shape
             parameters are reordered to be perfectly (anti)correlated with
             the peak height.
+        kwargs
+            Arbitrary keyword arguments, passed through to the constructor.
 
         Returns
         -------
-        problem : MultiplePeaksModel2 instance
+        problem : MultiplePeaksModel2
 
         """
         assert shape_height_correlation in (-1, 0, 1)
@@ -472,20 +478,20 @@ class MultiplePeaksModel2(TestProblem):
 
         Returns
         -------
-        objective_value : list of one float
+        objective_value : float
 
         """
         assert len(phenome) == self.num_variables
         g = self.g
         phenome = np.array(phenome)
         max_g_value = max(g(phenome, peak) for peak in self.peaks)
-        return [1.0 - max_g_value]
+        return 1.0 - max_g_value
 
 
     def get_active_peak(self, phenome):
         """Return the peak modeling the function at `phenome`."""
         g = self.g
-        max_objective_value = -1.0
+        max_objective_value = float("-inf")
         active_peak = None
         for peak in self.peaks:
             objective_value = g(phenome, peak)
@@ -513,6 +519,32 @@ class MultiplePeaksModel2(TestProblem):
         return current_peak
 
 
+    def get_peaks_sorted_by_importance(self):
+        """Return all peaks together with importance information.
+
+        In this implementation, the importance of a peak is defined as the
+        difference between a peak's height and the best value of other peak
+        functions at this location.
+
+        Returns
+        -------
+        decorated_peaks : list of tuple
+            A list of tuples (importance, peak) in descending order from
+            high to low importance. Negative values indicate that the peak
+            is not a local optimum.
+
+        """
+        g = self.g
+        decorated_peaks = []
+        for peak in self.peaks:
+            this_g_value = g(peak, peak)
+            other_g_values = [g(peak, other) for other in self.peaks if other is not peak]
+            importance = this_g_value - max(other_g_values)
+            decorated_peaks.append((importance, peak))
+        decorated_peaks.sort(reverse=True)
+        return decorated_peaks
+
+
     def get_locally_optimal_solutions(self, max_number=None):
         """Return locally optimal solutions (includes global ones).
 
