@@ -22,10 +22,11 @@ class MultiObjectiveTestProblem(TestProblem):
     def sample_pareto_front(self, num_points, oversampling_factor=100):
         """Sample the whole Pareto-front.
 
-        This method works by sampling ``num_points * oversampling_factor``
-        Pareto-optimal points in the search space, evaluating them, and
-        then selecting num_points of them with a uniform distribution in
-        objective space by the algorithm in
+        This method works by obtaining ``num_points * oversampling_factor``
+        Pareto-optimal points in the search space from
+        :func:`get_optimal_solutions<optproblems.base.TestProblem.get_optimal_solutions>`,
+        evaluating them, and then selecting num_points of them with a
+        uniform distribution in objective space by the algorithm in
         :func:`diversipy.subset.psa_partition`.
 
         Parameters
@@ -34,7 +35,8 @@ class MultiObjectiveTestProblem(TestProblem):
             The number of points to sample on the Pareto-front.
         oversampling_factor : int or float, optional
             A parameter controlling the uniformity of the points'
-            distribution on the Pareto-front.
+            distribution on the Pareto-front. The higher this value, the
+            higher is the chance of obtaining a uniform distribution.
 
         Returns
         -------
