@@ -2,7 +2,7 @@
 This module contains the test problems for multiobjective optimization
 published by the Walking Fish Group (WFG) [Huband2006]_. The problems here
 are a reimplementation of the original C++ code at
-http://www.wfg.csse.uwa.edu.au/toolkit/ .
+http://www.wfg.csse.uwa.edu.au/toolkit/ . They are all minimization problems.
 """
 
 import math
@@ -134,9 +134,9 @@ def s_decept(y, a, b, c):
     tmp1 = math.floor(y - a + b) * (1.0 - c + (a - b) / b) / (a - b)
     tmp2 = math.floor(a + b - y) * (1.0 - c + (1.0 - a - b) / b) / (1.0 - a - b)
     return correct_to_01(1.0 + (abs(y - a) - b) * (tmp1 + tmp2 + 1.0 / b))
-  
-  
-  
+
+
+
 def s_multi(y, a, b, c):
     """Transformation function."""
     assert y >= 0.0
@@ -148,7 +148,8 @@ def s_multi(y, a, b, c):
     assert c < 1.0
     tmp1 = abs(y - c) / (2.0 * (math.floor(c - y) + c))
     tmp2 = (4.0 * a + 2.0) * math.pi * (0.5 - tmp1)
-    return correct_to_01((1.0 + math.cos(tmp2) + 4.0 * b * math.pow(tmp1, 2.0)) / (b + 2.0))
+    result = (1.0 + math.cos(tmp2) + 4.0 * b * math.pow(tmp1, 2.0)) / (b + 2.0)
+    return correct_to_01(result)
 
 
 
@@ -182,7 +183,7 @@ def r_nonsep(y, a):
     denominator = len(y) * tmp * (1.0 + 2.0 * a - 2.0 * tmp) / a
     return correct_to_01(numerator / denominator)
 
-    
+
 
 def linear(x, m):
     """Shape function."""
@@ -201,9 +202,9 @@ def convex(x, m):
     assert shape_args_ok(x, m)
     result = 1.0
     for i in range(1, len(x) - m + 1):
-        result *= 1.0 - math.cos(x[i - 1] * math.pi / 2.0 ) 
+        result *= 1.0 - math.cos(x[i - 1] * math.pi / 2.0)
     if m != 1:
-        result *= 1.0 - math.sin(x[len(x) - m] * math.pi / 2.0 )   
+        result *= 1.0 - math.sin(x[len(x) - m] * math.pi / 2.0)
     return correct_to_01(result)
 
 
@@ -211,11 +212,11 @@ def convex(x, m):
 def concave(x, m):
     """Shape function."""
     assert shape_args_ok(x, m)
-    result = 1.0;
+    result = 1.0
     for i in range(1, len(x) - m + 1):
         result *= math.sin(x[i - 1] * math.pi / 2.0)
     if m != 1:
-        result *= math.cos(x[len(x) - m] * math.pi / 2.0)  
+        result *= math.cos(x[len(x) - m] * math.pi / 2.0)
     return correct_to_01(result)
 
 
@@ -227,7 +228,8 @@ def mixed(x, a, alpha):
     assert a >= 1
     assert alpha > 0.0
     tmp = 2.0 * a * math.pi
-    return correct_to_01(math.pow(1.0 - x[0] - math.cos(tmp * x[0] + math.pi / 2.0) / tmp, alpha))
+    result = math.pow(1.0 - x[0] - math.cos(tmp * x[0] + math.pi / 2.0) / tmp, alpha)
+    return correct_to_01(result)
 
 
 
@@ -239,7 +241,8 @@ def disc(x, a, alpha, beta):
     assert alpha > 0.0
     assert beta > 0.0
     tmp1 = a * math.pow(x[0], beta) * math.pi
-    return correct_to_01(1.0 - math.pow(x[0], alpha) * math.pow(math.cos(tmp1), 2.0))
+    result = 1.0 - math.pow(x[0], alpha) * math.pow(math.cos(tmp1), 2.0)
+    return correct_to_01(result)
 
 
 
@@ -259,7 +262,7 @@ class Shape:
 
     @staticmethod
     def normalize_z(z, z_max):
-        result = []     
+        result = []
         for i in range(0, len(z)):
             assert z[i] >= 0.0
             assert z[i] <= z_max[i]
@@ -278,7 +281,7 @@ class Shape:
             assert a[i] == 0 or a[i] == 1
             tmp1 = max(tp[-1], a[i])
             result.append(tmp1 * (tp[i] - 0.5) + 0.5)
-        result.append(tp[-1])    
+        result.append(tp[-1])
         return result
 
 
@@ -289,11 +292,11 @@ class Shape:
         assert len(x) == len(h)
         s = []
         for m in range(1, len(h) + 1):
-            s.append(m * 2.0)   
+            s.append(m * 2.0)
         return calculate_f(1.0, x, h, s)
 
 
-    def __call__(self):
+    def __call__(self, tp):
         raise NotImplementedError("Abstract class `Shape` is not callable.")
 
 
@@ -310,14 +313,14 @@ class WFGBaseProblem(MultiObjectiveTestProblem):
         assert num_objectives > 1
         assert k >= 1 and k < num_variables
         assert k % (num_objectives - 1) == 0
-        # maxObjectiveLimits = None
-        # minObjectiveLimits = None
-        # if num_objectives == 3:
-        #     maxObjectiveLimits = [10, 10, 10]
-        #     minObjectiveLimits = [0, 0, 0]
-        # elif num_objectives == 5:
-        #     maxObjectiveLimits = [10, 10, 10, 10, 12]
-        #     minObjectiveLimits = [0, 0, 0, 0, 0]
+        self.max_objectives = None
+        self.min_objectives = None
+        if num_objectives == 3:
+            self.max_objectives = [10.0, 10.0, 10.0]
+            self.min_objectives = [0.0, 0.0, 0.0]
+        elif num_objectives == 5:
+            self.max_objectives = [10.0, 10.0, 10.0, 10.0, 12.0]
+            self.min_objectives = [0.0, 0.0, 0.0, 0.0, 0.0]
         self.min_bounds = [0.0] * num_variables
         self.max_bounds = [2.0 * i for i in range(1, num_variables + 1)]
         MultiObjectiveTestProblem.__init__(self,
@@ -327,6 +330,7 @@ class WFGBaseProblem(MultiObjectiveTestProblem):
         self.k = k
         self.num_variables = num_variables
         self.is_deterministic = True
+        self.do_maximize = False
         if num_objectives <= 4:
             self.default_reference_set_size = 500
         else:
@@ -374,7 +378,7 @@ class WFGBaseProblem(MultiObjectiveTestProblem):
         params = []
         if rand_generator is None:
             rand_generator = random
-        for i in range(k):
+        for _ in range(k):
             params.append(rand_generator.random())
         return self.optimal_solution(k, l, params)
 
@@ -383,10 +387,13 @@ class WFGBaseProblem(MultiObjectiveTestProblem):
         solutions = []
         rand_generator = random.Random()
         rand_generator.seed(2)
-        l = self.num_variables - self.k
+        optimal_solution = self.optimal_solution
+        rand_optimal_solution = self.rand_optimal_solution
+        k = self.k
+        l = self.num_variables - k
         # generate full factorial sample of position parameters
         param_combis = [[]]
-        for i in range(self.k):
+        for _ in range(k):
             new_combis = []
             for combi in param_combis:
                 combi_copy = copy.deepcopy(combi)
@@ -396,10 +403,10 @@ class WFGBaseProblem(MultiObjectiveTestProblem):
             param_combis.extend(new_combis)
         # build solutions for these parameters
         for pos_params in param_combis:
-            solutions.append(self.optimal_solution(self.k, l, pos_params))
+            solutions.append(optimal_solution(k, l, pos_params))
         # fill up with random optimal solutions
         while len(solutions) < max_number:
-            solutions.append(self.rand_optimal_solution(self.k, l, rand_generator))
+            solutions.append(rand_optimal_solution(k, l, rand_generator))
         # assert length is not exceeded
         return solutions[0:max_number]
 
@@ -439,7 +446,7 @@ class WFG1(WFGBaseProblem):
         params = []
         if rand_generator is None:
             rand_generator = random
-        for i in range(k):
+        for _ in range(k):
             # account for polynomial bias
             params.append(pow(rand_generator.random(), 50.0))
         return self.optimal_solution(k, l, params)
@@ -447,7 +454,7 @@ class WFG1(WFGBaseProblem):
 
     @staticmethod
     def transition1(y, k):
-        n = len(y)   
+        n = len(y)
         assert vector_in_01(y)
         assert k >= 1
         assert k < n
@@ -473,7 +480,7 @@ class WFG1(WFGBaseProblem):
     def transition3(y):
         n = len(y)
         assert vector_in_01(y)
-        t = []    
+        t = []
         for i in range(0, n):
             t.append(b_poly(y[i], 0.02))
         return t
@@ -487,7 +494,7 @@ class WFG1(WFGBaseProblem):
         assert k < n
         assert m >= 2
         assert k % (m - 1) == 0
-        w = []   
+        w = []
         for i in range(1, n + 1):
             w.append(2.0 * i)
         t = []
@@ -848,7 +855,7 @@ class WFG8(WFGBaseProblem):
         # calculate the distance parameters
         for i in range(k, k + l):
             w = [1.0] * len(result)
-            u = r_sum(result, w )
+            u = r_sum(result, w)
             tmp1 = abs(math.floor(0.5 - u) + 0.98 / 49.98)
             tmp2 = 0.02 + 49.98 * (0.98 / 49.98 - (1.0 - 2.0 * u) * tmp1)
             result.append(pow(0.35, pow(tmp2, -1.0)))
@@ -970,9 +977,9 @@ class WFG9(WFGBaseProblem):
 class WFG(list):
     """The test problem collection WFG as defined in [Huband2006]_.
 
-     This class inherits from :class:`list` and fills itself with the nine
-     WFG problems with the chosen configuration. The arguments to the
-     constructor are passed through to the problem classes.
+    This class inherits from :class:`list` and fills itself with the nine
+    WFG problems with the chosen configuration. The arguments to the
+    constructor are passed through to the problem classes.
 
     References
     ----------
@@ -994,10 +1001,10 @@ class WFG(list):
         k : int
             The number of position related parameters, which are a part
             of the phenome. Thus it must hold ``k < num_variables``.
-            Furthermore, k must be a multiple of ``num_objectives - 1``
+            Furthermore, k must be a multiple of ``num_objectives - 1``.
             Huband et al. recommend ``k = 4`` for two objectives and
             ``k = 2 * (m - 1)`` for m objectives.
-        **kwargs
+        kwargs
             Arbitrary keyword arguments, passed through to the constructors
             of the single WFG problems.
 
